﻿using System;
using System.Windows.Forms;
using AIOUSBNet;

namespace Sample1
{
    public partial class MainWindow : Form
    {
        /// <summary>
        /// Specifies the device index to use.
        /// </summary>
        /// <remarks>
        /// USB devices are plug-and-play. Every time the driver detects a new USB device it assigns
        /// it a new, unused, Device Index. This is a temporary ID, and will be different each time
        /// any given hardware device is detected. NOTE: When using a single USB device, we can
        /// choose to ignore the specific Device Index value assigned by the driver, and instead
        /// refer to the device by the constant "diOnly" such as is done here. This value tells the
        /// driver "Whichever device index exists, that's the one I want to talk with." This is the
        /// simplest way to code, but not future-proof; if you add a second device later, you'll need
        /// to change the code. Here, we use a C# field in order to hold this value, so that then it
        /// is portable across this whole sample.
        /// </remarks>
        private const uint _deviceIndex = AIOUSB.diOnly;

        /// <summary>
        /// Treated as a Boolean flag: TRUE causes all bis on the device to enter tristate, i.e.,
        /// high-impedance, mode. FALSE removes the tristate. This value is used globally in all API
        /// calls in this sample.
        /// </summary>
        /// <remarks>
        /// The tristate is changed after the remainder of the configuration has occurred. All
        /// devices with this feature power-on in the "tristate" mode at ths time.
        /// </remarks>
        private const byte _tristateAll = 0;

        /// <summary>
        /// Specifies the number of pins on the device.
        /// </summary>
        private const uint NumPins = 48;

        /// <summary>
        /// Specifies the number of ports on the device.
        /// </summary>
        private const uint NumPorts = 6;

        /// <summary>
        /// Holds a reference to an array of controls that we use for our GUI updates.
        /// </summary>
        private readonly CheckBox[] _checkStates = new CheckBox[48];

        /// <summary>
        /// Data to send to/read from the device. Has one element for each port.
        /// </summary>
        private readonly byte[] _data = new byte[6];

        /// <summary>
        /// Defines the input/output mask for ports 0-7 (first element) and ports 8-11 (second
        /// element). Ports are numbered starting from zero.
        /// </summary>
        /// <remarks>
        /// The output mask is treated as an array of bits, one bit per I/O group. Each "1" bit in
        /// the array indicates that the corresponding I/O group of the device is to be configured as
        /// Output. For the USB-DIO-48 product, there are 6 I/O groups.
        /// </remarks>
        private byte[] _outMask = new byte[2];

        /// <summary>
        /// Constructs a new instance of <see cref="MainWindow"/>.
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();
        }

        /// <summary>
        /// This method is called to load the initial state of the form. We load the initial states
        /// of all the check boxes (pin state indicators) and initialize the arrays that need it.
        /// </summary>
        private void MainWindow_Load(object sender, EventArgs e)
        {
            // Initialize the array of all the LED-style checkboxes. Set each element of the
            // _checkStates array to the corresponding CheckBox control.

            // The board has two 50-pin headers, numbered 0-1. Each has 3 I/O Groups (ports), 
            // designated A, B, and C. Each I/O Group has 8 bits, numbered 0-7.
            _checkStates[0] = checkBoxA0_1;
            _checkStates[1] = checkBoxA0_2;
            _checkStates[2] = checkBoxA0_3;
            _checkStates[3] = checkBoxA0_4;
            _checkStates[4] = checkBoxA0_5;
            _checkStates[5] = checkBoxA0_6;
            _checkStates[6] = checkBoxA0_7;
            _checkStates[7] = checkBoxA0_8;
            _checkStates[8] = checkBoxB0_9;
            _checkStates[9] = checkBoxB0_10;
            _checkStates[10] = checkBoxB0_11;
            _checkStates[11] = checkBoxB0_12;
            _checkStates[12] = checkBoxB0_13;
            _checkStates[13] = checkBoxB0_14;
            _checkStates[14] = checkBoxB0_15;
            _checkStates[15] = checkBoxB0_16;
            _checkStates[16] = checkBoxC0_17;
            _checkStates[17] = checkBoxC0_18;
            _checkStates[18] = checkBoxC0_19;
            _checkStates[19] = checkBoxC0_20;
            _checkStates[20] = checkBoxC0_21;
            _checkStates[21] = checkBoxC0_22;
            _checkStates[22] = checkBoxC0_23;
            _checkStates[23] = checkBoxC0_24;

            _checkStates[24] = checkBoxA1_25;
            _checkStates[25] = checkBoxA1_26;
            _checkStates[26] = checkBoxA1_27;
            _checkStates[27] = checkBoxA1_28;
            _checkStates[28] = checkBoxA1_29;
            _checkStates[29] = checkBoxA1_30;
            _checkStates[30] = checkBoxA1_31;
            _checkStates[31] = checkBoxA1_32;
            _checkStates[32] = checkBoxB1_33;
            _checkStates[33] = checkBoxB1_34;
            _checkStates[34] = checkBoxB1_35;
            _checkStates[35] = checkBoxB1_36;
            _checkStates[36] = checkBoxB1_37;
            _checkStates[37] = checkBoxB1_38;
            _checkStates[38] = checkBoxB1_39;
            _checkStates[39] = checkBoxB1_40;
            _checkStates[40] = checkBoxC1_41;
            _checkStates[41] = checkBoxC1_42;
            _checkStates[42] = checkBoxC1_43;
            _checkStates[43] = checkBoxC1_44;
            _checkStates[44] = checkBoxC1_45;
            _checkStates[45] = checkBoxC1_46;
            _checkStates[46] = checkBoxC1_47;
            _checkStates[47] = checkBoxC1_48;

            // Iterate over the number of ports and set all elements of the _data array to 0xFF which
            // indicates "pulled up", or "high"
            for (var portIndex = 0; portIndex < NumPorts; portIndex++)
            {
                _data[portIndex] = 0xFF;
            }

            _outMask[0] = 0x00; // ports 0-7 as "input"
            _outMask[1] = 0x00; // ports 8-11 as "input"

            // Configure the board with initial settings.
            AIOUSB.DIO_Configure(_deviceIndex, _tristateAll, _outMask.Length, _outMask, _data.Length, _data);
        }

        /// <summary>
        /// Toggles one bit of data in the _data array. Which bit to toggle is selected by the "tag" field of the clicked checkbox, 
        /// which we configured in the Designer to hold the "Bit Index" appropriate to each checkbox's respective DIO pin.
        /// Note: AIOUSB.DIO_Write1() will be ignored if the bit index is an input.
        /// </summary>
        private void pinCheckBox_Click(object sender, EventArgs e)
        {
            var bitIndex = Convert.ToInt32(((CheckBox)sender).Tag);
            var bByte = (((_data[bitIndex / 8]) & (1 << (bitIndex % 8))) != 0) ? (byte)0 : (byte)1;

            // Toggle the state of the pin that corresponds to the check box that has been clicked.
            AIOUSB.DIO_Write1(_deviceIndex, (uint)bitIndex, bByte);  // byte used like a bool
        }

        /// <summary>
        /// This method is called each time the interval for the timer we have set up expires.
        /// NOTE: This functionality should be treated as if it is executing in a thread other than
        /// the main GUI thread.
        /// </summary>
        private void theTimer_Tick(object sender, EventArgs e)
        {
            // First, get the current state of the device's digital I/O pins.
            AIOUSB.DIO_ReadAll(_deviceIndex, _data.Length, _data);

            // This loop checks if a data bit reported a 1. It generates a bitmask (1 << (i % 8)) for
            // each of the 6 data bytes (_data[i / 8]) and checks to see if the corresponding bit is
            // set. The use of the / and % operators allows us to iterate over all bit indices while
            // addressing byte-wide elements of the _data array. If the value is nonzero then the
            // digital bit reported a 1; otherwise, zero is present.
            for (var pinIndex = 0; pinIndex < NumPins; pinIndex++)
            {
                _checkStates[pinIndex].Checked = ((_data[pinIndex / 8] & (1 << (pinIndex % 8))) != 0); // looks complicated but efficient
            }
        }

        /// <summary>
        /// This method is called when the "Toggle to Input Mode" button is clicked for a given port.
        /// </summary>
        private void toInputButton_Click(object sender, EventArgs e)
        {
            // get the tag index that is saved in the Tag property of the C# Button object. this
            // identifies which I/O group the clicked button pertains to.
            var portIndex = Convert.ToInt16(((Button)sender).Tag);

            // We are trying to toggle a specific I/O group to input mode. Each toggle button is
            // associated with the corresponding I/O group, as shown on the GUI. We get the index
            // that identifies *which* I/O bit by reading the Tag property of the C# Button object
            // who is the "sender" of this event, as identified above. So, what we need to do is go
            // into the output mask, find the bit corresponding to this I/O group, and then turn said
            // bit to "off" i.e., "0", which toggles the I/O group to which the button that
            // was clicked applies from "output" to the "input" state.
            var outputMaskValue = BitConverter.ToUInt16(_outMask, 0);
            outputMaskValue &= Convert.ToUInt16(0xFFFF & ~(1 << portIndex));
            _outMask = BitConverter.GetBytes(outputMaskValue);

            // some code to implement the button's "toggle direction" functionality
            ((Button)sender).Text = "Toggle to Output Mode";
            ((Button)sender).Click -= toInputButton_Click;
            ((Button)sender).Click += toOutputButton_Click;

            // Call DIO_Configure to actually configure the specified I/O group as input.
            AIOUSB.DIO_Configure(_deviceIndex, _tristateAll, _outMask.Length, _outMask, _data.Length, _data);
        }

        /// <summary>
        /// This method is called when the "Toggle to Output Mode" button is clicked for a given port.
        /// </summary>
        private void toOutputButton_Click(object sender, EventArgs e)
        {
            // get the tag index that is saved in the Tag property of the C# Button object. this
            // identifies which I/O group the clicked button pertains to.
            var portIndex = Convert.ToInt16(((Button)sender).Tag);

            // We are trying to toggle a specific I/O group to output mode. Each toggle button is
            // associated with the corresponding I/O group, as shown on the GUI. We get the index
            // that identifies *which* I/O bit by reading the Tag property of the C# Button object
            // who is the "sender" of this event, as identified above. So, what we need to do is go
            // into the output mask, find the bit corresponding to this I/O group, and then turn said
            // bit to "on" i.e., "1," which toggles the particular I/O group to which the button that
            // was clicked applies from "input" to the "output" state.
            var outputMaskValue = BitConverter.ToUInt16(_outMask, 0);
            outputMaskValue |= Convert.ToUInt16(1 << portIndex);

            // Now, as from previous, we get the outputMaskValue represented as an array of bytes and
            // store them in the _outMask value.
            _outMask = BitConverter.GetBytes(outputMaskValue);

            // some code to implement the button's "toggle direction" functionality
            ((Button)sender).Text = "Toggle to Input Mode";
            ((Button)sender).Click -= toOutputButton_Click;
            ((Button)sender).Click += toInputButton_Click;

            // Call DIO_Configure to actually configure the specified I/O group as output.
            AIOUSB.DIO_Configure(_deviceIndex, _tristateAll, _outMask.Length, _outMask, _data.Length, _data);
        }
    }
}